/*
     This file is part of GNUnet
     (C) 2005, 2006, 2010 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file src/fs/gnunet-fs-gtk-edit_publish_dialog.h
 * @author Christian Grothoff
 */
#ifndef GNUNET_FS_GTK_EDIT_PUBLISH_DIALOG_H
#define GNUNET_FS_GTK_EDIT_PUBLISH_DIALOG_H

#include "gnunet-fs-gtk-common.h"
#include <gnunet/gnunet_util_lib.h>
#include <gnunet/gnunet_fs_service.h>


/**
 * Function called when the edit publish dialog has been closed.
 *
 * @param cls closure
 * @param do_index index flag set?
 * @param short_fn short filename
 * @param anonymity_level anonymity degree chosen for publishing
 * @param priority replication setting (rename!)
 * @param root namespace root, NULL for file publishing
 * @param ret GTK_RESPONSE_OK if the dialog was closed with "OK"
 */
typedef void (*GNUNET_FS_GTK_EditPublishDialogCallback) (gpointer cls,
                                                         int do_index,
                                                         const char *short_fn,
                                                         guint anonymity_level,
                                                         guint priority,
                                                         const char *root,
                                                         int ret);


/**
 * Open the dialog to edit file information data.
 */
void
GNUNET_FS_GTK_edit_publish_dialog (GtkBuilder * builder, GtkWindow * parent,
                                   int do_index, const char *short_fn,
                                   uint32_t anonymity_level, uint32_t priority,
                                   struct GNUNET_FS_FileInformation *fip,
                                   gboolean allow_no_keywords,
                                   GNUNET_FS_GTK_EditPublishDialogCallback cb,
                                   gpointer cls);

#endif
/* end of gnunet-fs-gtk-edit_publish_dialog.h */
