/*
     This file is part of GNUnet
     (C) 2005, 2006, 2010 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file src/fs/gnunet-fs-gtk-main_window_create_pseudonym.c
 * @author Christian Grothoff
 *
 */
#include "gnunet-fs-gtk-common.h"
#include "gnunet-fs-gtk.h"

void
GNUNET_GTK_create_namespace_dialog_response_cb (GtkDialog * dialog,
                                                gint response_id,
                                                gpointer user_data)
{
  const char *name;
  gchar *name_loc;
  struct GNUNET_FS_Namespace *ns;
  GtkWidget *ad;
  GtkBuilder *builder;

  builder = GTK_BUILDER (user_data);
  ad = GTK_WIDGET (gtk_builder_get_object
                   (builder, "GNUNET_GTK_create_namespace_dialog"));

  if (response_id != -5)
  {
    gtk_widget_destroy (ad);
    g_object_unref (G_OBJECT (builder));
    return;
  }
  name =
      gtk_entry_get_text (GTK_ENTRY
                          (gtk_builder_get_object
                           (builder,
                            "GNUNET_GTK_create_namespace_name_entry")));
  /* FIXME: show busy dialog while doing key creation */
  name_loc = GNUNET_GTK_from_utf8_to_loc ((char *) name);
  if (NULL != name_loc)
  {
    ns = GNUNET_FS_namespace_create (GNUNET_FS_GTK_get_fs_handle (), name_loc);
    GNUNET_FS_namespace_delete (ns, GNUNET_NO);
    GNUNET_free (name_loc);
  }
  gtk_widget_destroy (ad);
  g_object_unref (G_OBJECT (builder));
}

/**
 */
void
GNUNET_GTK_main_menu_create_pseudonym_activate_cb (GtkWidget * dummy,
                                                   gpointer data)
{
  GtkWidget *ad;
  GtkWidget *toplevel;
  GtkBuilder *builder;

  builder =
      GNUNET_GTK_get_new_builder
      ("gnunet_fs_gtk_create_namespace_dialog.glade");
  if (builder == NULL)
  {
    GNUNET_break (0);
    return;
  }
  ad = GTK_WIDGET (gtk_builder_get_object
                   (builder, "GNUNET_GTK_create_namespace_dialog"));

  toplevel = gtk_widget_get_toplevel (dummy);
  if (GTK_IS_WINDOW (toplevel))
    gtk_window_set_transient_for (GTK_WINDOW (ad), GTK_WINDOW (toplevel));
  gtk_window_present (GTK_WINDOW (ad));
}


/* end of gnunet-fs-gtk-main_window_create_pseudonym.c */
